;*****************************************************************************
;
; TreeProcess
; Copyright 2014 Jsahel Benoist
;
; --- License ---
;
; This file is part of TreeProcess
;                                                                   
; TreeProcess is free software: you can redistribute it and/or modify 
; it under the terms of the GNU General Public License as publish
; the Free Software Foundation, either version 3 of the License, 
; (at your option) any later version.                            
;                                                                   
; TreeProcess is distributed in the hope that it will be useful,      
; but WITHOUT ANY WARRANTY; without even the implied warranty of 
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
; GNU General Public License for more details.                   
;                                                                   
; You should have received a copy of the GNU General Public Licence
; along with TreeProcess.  If not, see <http://www.gnu.org/licenses/>.
;
; --- Description ---
;
; - English -                                                                                                                                                                                                                           
; This program walks the file tree and execute specified command on 
; found files and/or folders. Useful to batch without coding, 
; eg. converting *.wav to mp3, *.bmp To *.jpg, and so on...                                                     ed by
; Somewhat like the UNIX 'find' command, with a GUI.

; - French -                                                                                                                                                                                                                            
; Ce programme parcourt l'arborescence et excute une commande 
; spcifie sur les fichiers et/ou dossiers trouvs. Utile pour 
; convertir  la vole (batch processing) sans avoir  programmer ! 
; Par exemple *.wav en mp3, *.bmp en jpeg, etc.
;
; --- Compilation detail ---
;
; Compiled on PureBasic 5.22 (c)Fantaisie Software
;
; --- History ---
;
; July, the 25th of 2014 : Initial release by Jsahel Benoist
; 26-28 : Added duplication of source directory, config save/file, 
;         ImageConverter.exe command example, %APPDIR% for command path
;
;*****************************************************************************

CompilerIf #PB_Compiler_OS = #PB_OS_Windows
	
	#PATH_SEPARATOR = "\"	
	
	Procedure.s SpecialFolder(SpecialFolder)
		
		Protected ListPtr.l, SpecialFolderPath.s
		
		SpecialFolderPath = Space(#MAX_PATH)
		If SHGetSpecialFolderLocation_(0, SpecialFolder, @ListPtr) = #NOERROR
			SHGetPathFromIDList_(ListPtr, @SpecialFolderPath)
		EndIf
		ProcedureReturn Trim(SpecialFolderPath)
		
	EndProcedure
	
	CompilerElseIf #PB_Compiler_OS = #PB_OS_MacOS
	
	#PATH_SEPARATOR = "/"	
	
	Procedure.s SpecialFolder(CSIDL)
		ProcedureReturn GetHomeDirectory() + "Library/Application Support/"
	EndProcedure
	
CompilerElse
	
	#PATH_SEPARATOR = "/"	
	
CompilerEndIf

EnableExplicit

XIncludeFile "MainWindow.pbf" ; Window interface

;*****************************************************************************
;- PROCEDURES 
;*****************************************************************************

Procedure ButtonPathSelectEvent(EventType)
	Define Path$
	Path$ = PathRequester("Please select a path", GetPathPart(ProgramFilename()))
	If Path$ <> ""
		SetGadgetText(StringPath, Path$)
	EndIf
EndProcedure

;*****************************************************************************

Procedure ButtonSelectCommandEvent(EventType)
	Define Command$
	Command$ = OpenFileRequester("Please choose some exe file", GetPathPart(ProgramFilename()), "Exe files|*.exe|All files|*.*", 0)
	If Command$ <> ""
		SetGadgetText(StringCommand, Command$)
	EndIf
EndProcedure  

;*****************************************************************************

Procedure ButtonDestinationSelectEvent(EventType)
	Define Path$
	Path$ = PathRequester("Please select a path where duplicating tree", GetPathPart(ProgramFilename()))
	If Path$ <> ""
		SetGadgetText(StringDestination, Path$)
	EndIf
EndProcedure

;*****************************************************************************

Procedure ButtonFullPathEvent(EventType)
	SetGadgetText(StringParameters, GetGadgetText(StringParameters) + " %f ")
EndProcedure

;*****************************************************************************

Procedure ButtonPathOnlyEvent(EventType)
	SetGadgetText(StringParameters, GetGadgetText(StringParameters) + " %p ")
EndProcedure

;*****************************************************************************

Procedure ButtonFilenameOnlyEvent(EventType)
	SetGadgetText(StringParameters, GetGadgetText(StringParameters) + " %n ")
EndProcedure

;*****************************************************************************

Procedure ButtonExtensionEvent(EventType)
	SetGadgetText(StringParameters, GetGadgetText(StringParameters) + " %e ")
EndProcedure

;*****************************************************************************

Procedure ButtonHomeEvent(EventType)
	SetGadgetText(StringParameters, GetGadgetText(StringParameters) + " %h ")
EndProcedure

;*****************************************************************************

Procedure ButtonTempEvent(EventType)
	SetGadgetText(StringParameters, GetGadgetText(StringParameters) + " %t ")
EndProcedure

;*****************************************************************************

Procedure ButtonDupEvent(EventType)
	SetGadgetText(StringParameters, GetGadgetText(StringParameters) + " %d ")
EndProcedure

;*****************************************************************************

Procedure LoadConfig(ConfigFile$)
	
	If ConfigFile$ <> ""
		If OpenPreferences(ConfigFile$)
			SetGadgetText(StringPath, ReadPreferenceString("StringPath", ""))
			SetGadgetText(StringSearchText, ReadPreferenceString("StringSearchText", ""))
			SetGadgetState(CheckboxProcessFiles, ReadPreferenceInteger("CheckboxProcessFiles", #True))
			SetGadgetState(CheckboxProcessFolders, ReadPreferenceInteger("CheckboxProcessFolders", #False))
			SetGadgetText(StringDestination, ReadPreferenceString("StringDestination", ""))
			SetGadgetText(StringCommand, ReadPreferenceString("StringCommand", ""))
			SetGadgetText(StringParameters, ReadPreferenceString("StringParameters", ""))
			ClosePreferences()
		Else
			MessageRequester("Alert", "Could not open config file")
		EndIf
	EndIf
	
EndProcedure

;*****************************************************************************

Procedure ButtonLoadConfigEvent(EventType)
	
	Define ConfigFile$
	
	ConfigFile$ = SaveFileRequester("Please choose some config file", GetPathPart(ProgramFilename()) + "Config" + #PATH_SEPARATOR + "default.ini", "Config files (*.ini)|*.ini|All files|*.*", 0)
	LoadConfig(ConfigFile$)
	
EndProcedure

;*****************************************************************************

Procedure.l SaveConfig(ConfigFile$)
	
	If ConfigFile$ <> ""
		If CreatePreferences(ConfigFile$)
			WritePreferenceString("StringPath", GetGadgetText(StringPath))
			WritePreferenceString("StringSearchText", GetGadgetText(StringSearchText))
			WritePreferenceInteger("CheckboxProcessFiles", GetGadgetState(CheckboxProcessFiles))
			WritePreferenceInteger("CheckboxProcessFolders", GetGadgetState(CheckboxProcessFolders))
			WritePreferenceString("StringDestination", GetGadgetText(StringDestination))
			WritePreferenceString("StringCommand", GetGadgetText(StringCommand))
			WritePreferenceString("StringParameters", GetGadgetText(StringParameters))
			ClosePreferences()
			ProcedureReturn #True
		Else
			ProcedureReturn #False
		EndIf
	EndIf
	
EndProcedure

;*****************************************************************************

Procedure ButtonSaveConfigEvent(EventType)
	
	Define ConfigFile$
	
	ConfigFile$ = SaveFileRequester("Please choose some config file", GetPathPart(ProgramFilename()) + "Config" + #PATH_SEPARATOR + "default.ini", "Config files (*.ini)|*.ini|All files|*.*", 0)
	
	If SaveConfig(ConfigFile$)
		MessageRequester("Info", "Config file successfully saved.", #PB_MessageRequester_Ok)
	Else
		MessageRequester("Alert", "Could not create config file")
	EndIf
	
EndProcedure

;*****************************************************************************

Procedure LoadDefaultConfig(DefaultConfigPath$)
	
	If FileSize(DefaultConfigPath$) > 0
	  Debug "Load config " + DefaultConfigPath$
	  LoadConfig(DefaultConfigPath$) 
	Else  
	  Debug "Can't load config " + DefaultConfigPath$	
	EndIf
	
EndProcedure

;*****************************************************************************

Procedure SaveDefaultConfig(DefaultConfigPath$)
	
	;Try to save default config file. If not possible, try to create application appdata directory, and retry to save.
	If Not SaveConfig(DefaultConfigPath$)
	  Debug "Can't save config " + DefaultConfigPath$	
		If CreateDirectory(SpecialFolder(#CSIDL_APPDATA) + #PATH_SEPARATOR + "TreeProcess")
  	  Debug "->Create directory " + SpecialFolder(#CSIDL_APPDATA) + #PATH_SEPARATOR + "TreeProcess"	
  	  If SaveConfig(DefaultConfigPath$)
	  	  Debug "->Config saved to " + DefaultConfigPath$	
  	  Else
  	    Debug "->Still can't save config " + DefaultConfigPath$	
  	  EndIf
		EndIf
	Else
	  Debug "->Config saved to " + DefaultConfigPath$	
	EndIf
	
EndProcedure

;*****************************************************************************

Procedure CheckParameters()
	
	If CountString(GetGadgetText(StringParameters), "%d") = 0 And CountString(GetGadgetText(StringParameters), "%t") = 0 And CountString(GetGadgetText(StringParameters), "%h") = 0 And CountString(GetGadgetText(StringParameters), "%d") = 0 And CountString(GetGadgetText(StringParameters), "%f") = 0 And CountString(GetGadgetText(StringParameters), "%n") = 0 And CountString(GetGadgetText(StringParameters), "%e") = 0
		ProcedureReturn  #False
	Else
		ProcedureReturn  #True
	EndIf
	
EndProcedure

;*****************************************************************************
;Launch command on each file found and transfert data to the standard output
Procedure ExecuteCommand(Path$, DestPath$)
	
	Define Command$, Result, Output$, DataSize, Parameters$
	Define *Buffer
	
	Parameters$ = ReplaceString(GetGadgetText(StringParameters), "%f", Path$)
	Parameters$ = ReplaceString(Parameters$, "%p", GetPathPart(Path$))
	Parameters$ = ReplaceString(Parameters$, "%n", GetFilePart(Path$, #PB_FileSystem_NoExtension))
	Parameters$ = ReplaceString(Parameters$, "%e", GetExtensionPart(Path$))
	Parameters$ = ReplaceString(Parameters$, "%h", GetHomeDirectory())
	Parameters$ = ReplaceString(Parameters$, "%t", GetTemporaryDirectory())
	Parameters$ = ReplaceString(Parameters$, "%d", DestPath$)
	
	Command$ = GetGadgetText(StringCommand)
	;%APPDIR% is replaced by the TreeProcess path
	Command$ = ReplaceString(Command$, "%APPDIR%", GetPathPart(ProgramFilename()))

	Debug "Running " + Chr(34) + Command$ + " " + Parameters$ + Chr(34)
	Result = RunProgram(Command$, Parameters$, "", #PB_Program_Open | #PB_Program_Read)
	
		;-Not implemented : send data received by the program to the standard output.
 	If Result 
 	  While ProgramRunning(Result)
 			DataSize = AvailableProgramOutput(Result)
 			If DataSize > 0
 				*Buffer = AllocateMemory(4096)
 				While DataSize > 4096
 					ReadProgramData(Result, *Buffer, 4096)
 					WriteConsoleData(*Buffer, 4096)
 					DataSize - 4096
 				Wend
 				ReadProgramData(Result, *Buffer, DataSize)
 				WriteConsoleData(*Buffer, DataSize)
 				FreeMemory(*Buffer)
 			EndIf
 		Wend
 		CloseProgram(Result)
 	EndIf
	
EndProcedure

;*****************************************************************************

Procedure ButtonStartEvent(EventType)
	
	Define Path$, Dir.i, Type$, Size$, Name$, DestName$, FilesNb = 0, FoldersNB = 0, SrcPath$, DestPath$
	NewList DirList.s()
	
  OpenConsole()
	
	AddElement(Dirlist())
	
	If GetGadgetText(StringPath) = ""
		SrcPath$ = GetPathPart(ProgramFilename())
	Else
		SrcPath$ = GetGadgetText(StringPath)
	EndIf
	
	DestPath$ = GetGadgetText(StringDestination)
	
	;Try to explore the given tree without using recursivity to avoid stack problems
	ResetList(DirList()) ;Start (while/wend processing)
	While NextElement(DirList())
		Path$ = DirList()
		Debug "----"
		Debug "Current Path : " + SrcPath$ + Path$
		
		;First, look for folders to explore next
		Dir = ExamineDirectory(#PB_Any, SrcPath$ + Path$, "")
		If Dir			
			While NextDirectoryEntry(Dir)
				If DirectoryEntryName(Dir) <> "." And DirectoryEntryName(Dir) <> ".."
					If DirectoryEntryType(Dir) = #PB_DirectoryEntry_Directory
						PushListPosition(DirList());Save actual pos in tree
						AddElement(DirList())			 ;Add a new path to explore later
						DirList() = Path$ + DirectoryEntryName(Dir) + #PATH_SEPARATOR
						;Create the duplicated tree
						;(The folder checkbox doesn't need to be checked)
						If DestPath$ <> ""
  						DestName$ = DestPath$ + Path$ + DirectoryEntryName(Dir)
							Debug "Creating dir : " + DestName$
							If Not CreateDirectory(DestName$)
								Debug " -> Failed"	
							EndIf
						EndIf

						PopListPosition(DirList()) ;Back to the original pos
						;Debug "back to -" + DirList()
					EndIf
				EndIf
			Wend
			FinishDirectory(Dir)
			DeleteElement(DirList())  ;This folder is done
			;ResetList(DirList())	;Start over
		EndIf
		
		;Second, look for all entries according to filter
		Dir = ExamineDirectory(#PB_Any, SrcPath$ + Path$, GetGadgetText(StringSearchText))
		If Dir			
			While NextDirectoryEntry(Dir)
				If DirectoryEntryName(Dir) <> "." And DirectoryEntryName(Dir) <> ".."
					If DirectoryEntryType(Dir) = #PB_DirectoryEntry_File
						Type$ = " [Found file] "
						Size$ = " (Size : " + DirectoryEntrySize(Dir) + ")"
						Name$ = SrcPath$ + Path$ + DirectoryEntryName(Dir)
						DestName$ = DestPath$ + Path$
						Debug Type$ + Name$ + Size$
						If GetGadgetState(CheckboxProcessFiles) = #PB_Checkbox_Checked 
							;- Not implemented : file copy
							;							If DestPath$ <> ""
							;								Debug "Copying : " + Name$ + " to " + DestName$
							;								If Not CopyFile(Name$, DestName$)
							;									Debug " -> Failed"	
							;								EndIf
							;							EndIf
							If GetGadgetText(StringCommand) <> "" And CheckParameters() 
								ExecuteCommand(Name$, DestName$)
							EndIf
							FilesNb + 1
						EndIf
					Else
						Type$ = " [Found folder] "
						Size$ = ""
						Name$ = SrcPath$ + Path$ + DirectoryEntryName(Dir) + #PATH_SEPARATOR
						DestName$ = DestPath$ + Path$ + DirectoryEntryName(Dir)
						Debug Type$ + Name$ + Size$
						If GetGadgetText(StringCommand) <> "" And CheckParameters() And GetGadgetState(CheckboxProcessFolders) = #PB_Checkbox_Checked 
							ExecuteCommand(Name$, DestName$)
							FoldersNb + 1
						EndIf  
					EndIf
				EndIf
			Wend
			FinishDirectory(Dir)
		EndIf
		
	Wend	
	Debug "Total found files : " + Str(FilesNb)
	Debug "Total found folders : " + Str(FoldersNb)
	
	;CloseConsole()
	
EndProcedure

;*****************************************************************************
;- MAIN 
;*****************************************************************************

Define Event.i
Define DefaultConfigPath$ = SpecialFolder(#CSIDL_APPDATA) + #PATH_SEPARATOR + "TreeProcess" + #PATH_SEPARATOR + "default.ini"

OpenMainWindow()
LoadDefaultConfig(DefaultConfigPath$)

Repeat
	Event = WaitWindowEvent()
	
	Select EventWindow()
		Case MainWindow
			MainWindow_Events(Event)		
			
	EndSelect
	
Until Event = #PB_Event_CloseWindow

SaveDefaultConfig(DefaultConfigPath$)

End
; IDE Options = PureBasic 5.22 LTS (Windows - x64)
; CursorPosition = 42
; Folding = ----
; EnableXP
; CompileSourceDirectory